@@ -31,13 +31,13 @@ def serialize_path(path, append_last_slash=False):
 def index(path):
 	index, dir, ids = Files.Dictionary(path=False, dictionary={}), Files.Directory(path=path), []
 	for _path_ in dir.paths(recursive=True, files_only=True, banned=[gfp.clean(f"{path}/Icon\r")], banned_names=[".DS_Store", "__pycache__"]):
-		if _path_ not in ids: 
+		if _path_ not in ids and "/__pycache__/" not in path and "/.DS_Store" not in path: 
 			index[_path_] = gfp.mtime(path=_path_, format="seconds")
 			ids.append(_path_)
 	for _path_ in dir.paths(recursive=True, dirs_only=True, banned=[gfp.clean(f"{path}/Icon\r")], banned_names=[".DS_Store", "__pycache__"]):
 		id = _path_+" (d)"
 		if os.listdir(_path_) == []: id += " (e)"
-		if id not in ids: 
+		if id not in ids and "/__pycache__/" not in path and "/.DS_Store" not in path: 
 			index[id] = gfp.mtime(path=_path_, format="seconds")
 			ids.append(id)
 	return index.sort(alphabetical=True)
@@ -131,8 +131,8 @@ def pull(
 	delete_str = Formats.Boolean(delete).convert(true="--delete", false="")
 	lremote = serialize_path(gfp.clean(remote), append_last_slash=directory)
 	lpath = serialize_path(gfp.clean(path), append_last_slash=directory)
-	_command_ = f"rsync -{Formats.Boolean(directory).convert(true='a', false='')}zqt '{alias}:{lremote}' '{lpath}' {exclude_str} {include_str} {delete_str}"
-	if log_level >= 3: print(f"Rsync command: [{_command_}].")
+	_command_ = f"rsync -{Formats.Boolean(directory).convert(true='a', false='')}zqt '{alias}:{lremote}' '{lpath}' {exclude_str} {include_str} {delete_str} --timeout={SSH_TIMEOUT}"
+	#_command_ = f"rsync -azqtr '{alias}:{lremote}' '{lpath}' {exclude_str} {include_str} {delete_str}"
 
 	# execute.
 	if command: return _command_
@@ -142,6 +142,7 @@ def pull(
 			message=f"Successfully pulled [{alias}:{remote}] to [{path}].",
 			error=f"Failed to pull [{alias}:{remote}] to [{path}].",
 			loader=f"Pulling [{alias}:{remote}] to [{path}]",
+			log_level=log_level,
 		)
 
 	#
@@ -208,15 +209,12 @@ def push(
 		elif directory and not os.path.isdir(path):
 			return r3sponse.error_response(f"Path {path} is not a directory.")
 
-		"""
-		NOTE:
-		testing if check remote base can also be included if check==True only.
-		"""
-		# check remote base.
-		base = Formats.FilePath(remote).base(back=1)
-		response = utils.test_ssh_dir(alias=alias, path=base, accept_new_host_keys=accept_new_host_keys, create=True, checks=False)
-		if not response.success: return response
-		if response.created and log_level >= 3: print(f"Created remote directory {base}.")
+	# check remote base.
+	# must be excluded from the checks == False.
+	base = Formats.FilePath(remote).base(back=1)
+	response = utils.test_ssh_dir(alias=alias, path=base, accept_new_host_keys=accept_new_host_keys, create=True, checks=False)
+	if not response.success: return response
+	if response.created and log_level >= 3: print(f"Created remote directory {base}.")
 
 	# options.
 	exclude_str = Files.Array(array=exclude).string(joiner=" --exclude ", sum_first=True)
@@ -224,8 +222,8 @@ def push(
 	delete_str = Formats.Boolean(delete).convert(true="--delete", false="")
 	lremote = serialize_path(gfp.clean(remote), append_last_slash=directory)
 	lpath = serialize_path(gfp.clean(path), append_last_slash=directory)
-	_command_ = f"rsync -{Formats.Boolean(directory).convert(true='a', false='')}zqt '{lpath}' '{alias}:{lremote}' {exclude_str} {include_str} {delete_str}"
-	if log_level >= 3: print(f"Rsync command: [{_command_}].")
+	_command_ = f"rsync -{Formats.Boolean(directory).convert(true='a', false='')}zqt '{lpath}' '{alias}:{lremote}' {exclude_str} {include_str} {delete_str} --timeout={SSH_TIMEOUT}"
+	#_command_ = f"rsync -azqtr --rsh=ssh '{lpath}' '{alias}:{lremote}' {exclude_str} {include_str} {delete_str}"
 	
 	# execute.
 	if command: return _command_
@@ -235,6 +233,7 @@ def push(
 			message=f"Successfully pushed [{path}] to [{alias}:{remote}].",
 			error=f"Failed to push [{path}] to [{alias}:{remote}].",
 			loader=f"Pushing [{path}] to [{alias}:{remote}].",
+			log_level=log_level,
 		)
 
 	#
@@ -249,22 +248,55 @@ def execute(
 	error="Failed to execute the specified command.",
 	# loader message.
 	loader=None,
+	# serialize to json (overwrites message & error).
+	serialize=False,
+	# get the output.
+	get_output=False,
+	# the log level.
+	log_level=0,
 ):
 
 	# execute.
+	if log_level >= 6: print(command)
 	if loader != None:
 		loader = syst3m.console.Loader(loader, interactive=INTERACTIVE)
-	output = syst3m.utils.__execute_script__(command)
+	#output = syst3m.utils.__execute_script__(command)
+	path = f"/tmp/tmp_script_{Formats.String('').generate()}"
+	Files.save(path, command)
+	#try:
+	#	output = subprocess.check_output(["sh", path]).decode()
+	try:
+		output = subprocess.check_output(["sh", path]).decode()
+	except subprocess.CalledProcessError as e:
+		#if " exit status " in str(e.output):
+		#	return r3sponse.error_response(f"Failed to execute command [{command}], (exit status: {str(e.output).split(' exit status ')[1].split('.')[0].split(' ')[0]}).")
+		#else:
+		#	return r3sponse.error_response(f"Failed to execute command [{command}].")
+		return r3sponse.error_response(f"Failed to execute command [{command}], (output: {e.output}), (error: {e}).")
+	Files.delete(path)
 	if len(output) > 0 and output[len(output)-1] == "\n": output = output[:-1]
 
 	# handler.
-	if "rsync: " in output or "rsync error: " in output or "ssh: " in output:
+	response = utils.check_errors(output)
+	if not response.success:
 		if loader != None: loader.stop(success=False)
-		print(output)
-		return r3sponse.error_response(error)
+		return response
+		#print(output)
+		#return r3sponse.error_response(error)
 	else:
 		if loader != None: loader.stop()
-		return r3sponse.success_response(message)
+		if serialize:
+			try: response = r3sponse.serialize(output)
+			except Exception as e: 
+				return r3sponse.error_response(f"Failed to serialize output: {output}")
+			return response
+		else:
+			if get_output:
+				return r3sponse.success_response(message, {
+					"output":output,
+				})
+			else:
+				return r3sponse.success_response(message)
 
 	#
 
