@@ -13,10 +13,29 @@ def clean_path(path):
 	return path
 
 # test ssh functions.
+def check_errors(output):
+	for i in [
+		"rsync: ",
+		"rsync error: ",
+		"ssh: ",
+		"ssh error: ",
+		"client_loop: send disconnect: Broken pipe",
+	]:
+		if i in output:
+			e = Formats.String(i+output.split(i)[1].split("\n")[0]).capitalized_word().replace("Client_loop: send disconnect:", "Client loop: send disconnect:").replace("Ssh", "SSH")
+			while True:
+				if len(e) > 0 and e[len(e)-1] in [" ", "."]: e = e[:-1]
+				elif len(e) > 0 and e[0] in [" "]: e = e[1:]
+				else: break
+			return r3sponse.error_response(e+".")
+	return r3sponse.success_response("The output contains no (default) errors.")
 def test_ssh(alias=None, accept_new_host_keys=True, checks=True):
 	accept_new_host_keys = Formats.Boolean(accept_new_host_keys).convert(true="printf 'yes' | ", false="")
-	output = syst3m.utils.__execute_script__(f"""{accept_new_host_keys}ssh {alias} ' echo "Hello World" ' """)
-	if "Hello World" in output:
+	output = syst3m.utils.__execute_script__(f"""{accept_new_host_keys}ssh {DEFAULT_SSH_OPTIONS} {alias} ' echo "Hello World" ' """)
+	response = check_errors(output)
+	if not response.success:
+		return r3sponse.error_response(f"Failed to connect with {alias}, error: {output}")
+	elif "Hello World" in output:
 		return r3sponse.success_response(f"Successfully connected with {alias}.")
 	else:
 		return r3sponse.error_response(f"Failed to connect with {alias}, error: {output}")
@@ -24,8 +43,11 @@ def test_ssh_path(alias=None, path=None, accept_new_host_keys=True, checks=True)
 	if checks:
 		response = test_ssh(alias=alias, accept_new_host_keys=accept_new_host_keys)
 		if not response.success: return response
-	output = syst3m.utils.__execute_script__(f"""ssh {alias} ' ls -ld {path} ' """)
-	if "No such file or directory" not in output:
+	output = syst3m.utils.__execute_script__(f"""ssh {DEFAULT_SSH_OPTIONS} {alias} ' ls -ld {path} ' """)
+	response = check_errors(output)
+	if not response.success:
+		return r3sponse.error_response(f"Path {alias}:{path} does not exist.")
+	elif "No such file or directory" not in output:
 		return r3sponse.success_response(f"Path {alias}:{path} exists.")
 	else:
 		return r3sponse.error_response(f"Path {alias}:{path} does not exist.")
@@ -34,7 +56,7 @@ def test_ssh_dir(alias=None, path=None, accept_new_host_keys=True, create=False,
 		response = test_ssh(alias=alias, accept_new_host_keys=accept_new_host_keys)
 		if not response.success: return response
 	lpath = "\'"+path+"\'"
-	output = syst3m.utils.__execute_script__(f"""ssh {alias} ''' python3 /usr/local/lib/ssht00ls/classes/utils/isdir.py {path}''' """)
+	output = syst3m.utils.__execute_script__(f"""ssh {DEFAULT_SSH_OPTIONS} {alias} ''' python3 /usr/local/lib/ssht00ls/classes/utils/isdir.py {path}''' """)
 	if output.replace("\n","") == "directory":
 		return r3sponse.success_response(f"Path {alias}:{path} is a directory.", {
 			"created":created,
@@ -42,7 +64,7 @@ def test_ssh_dir(alias=None, path=None, accept_new_host_keys=True, create=False,
 	elif output.replace("\n","") in ["directory", "does-not-exist"]:
 		if output.replace("\n","") in ["does-not-exist"]:
 			if create:
-				output = syst3m.utils.__execute_script__(f"""ssh {alias} ''' mkdir -p {path}''' """)
+				output = syst3m.utils.__execute_script__(f"""ssh {DEFAULT_SSH_OPTIONS} {alias} ''' mkdir -p {path}''' """)
 				if "permission denied" in output: return r3sponse.error_response(f"Unable to create remote directory [{alias}:{path}].")
 				return test_ssh_dir(alias=alias, path=path, checks=False, create=False, created=True)
 			else:
@@ -58,7 +80,7 @@ def test_ssht00ls(alias=None, accept_new_host_keys=True, install=True):
 			if response.error == f"Path {alias}:{path} does not exist.":
 				if install:
 					loader = syst3m.console.Loader(f"Installing ssht00ls library on remote {alias}.")
-					output = syst3m.utils.__execute_script__(f"ssh {alias} ' curl https://raw.githubusercontent.com/vandenberghinc/{ALIAS}/master/{ALIAS}/requirements/installer.remote | bash ' ")
+					output = syst3m.utils.__execute_script__(f"ssh {DEFAULT_SSH_OPTIONS} {alias} ' curl https://raw.githubusercontent.com/vandenberghinc/{ALIAS}/master/{ALIAS}/requirements/installer.remote | bash ' ")
 					response = test_ssht00ls(alias=alias, accept_new_host_keys=accept_new_host_keys, install=False)
 					loader.stop(success=response.success)
 					print(output)
